@@ -8,7 +8,7 @@ use strict;
 use vars qw($VERSION);
 use IO::Socket::INET;
 
-$VERSION = sprintf("%d.%02d", q{Revision: 0.02} =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d.%02d", q{Revision: 0.03} =~ /(\d+)\.(\d+)/);
 
 sub new {
   my $this = shift();
@@ -22,17 +22,13 @@ sub new {
 sub initialize {
   my $self = shift();
   my %param = @_;
-  $self->port($param{'port'}         || '2947');
+  $self->port($param{'port'} || '2947');
   $self->name($param{'name'} || 'GPSD');
 }
 
 sub start {
   my $self=shift();
-  my %param = @_;
-  $self->lat($param{'lat'}           ||  39.5);
-  $self->lon($param{'lon'}           || -77.5);
-  $self->speed($param{'speed'}       ||  25.2);
-  $self->heading($param{'heading'}   ||  80.5);
+  my $provider=shift();
   $SIG{CHLD} = 'IGNORE';
   my $listen_socket = IO::Socket::INET->new(LocalPort=>$self->port,
                                             Listen=>10,
@@ -41,7 +37,7 @@ sub start {
 
   die "Can't create a listening socket: $@" unless $listen_socket;
 
-  while ($listen_socket->opened and my $connection = $listen_socket->accept) {
+  while ($listen_socket->opened and my $connection=$listen_socket->accept) {
     my $child;
     die "Can't fork: $!" unless defined ($child = fork());
     if ($child == 0) {       #i'm the child!
@@ -50,15 +46,15 @@ sub start {
       my $w=0;
       my $watcher_pid=undef();
       my $name=$self->name;
-      while (my $data=$connection->getline) {
-        next unless $data=~/\S/;       # blank line
-        if    ($data=~m/l/i) {
-          print $connection "$name,L=0 $VERSION lw Net-GPSD-Server-Fake\n";
-        } elsif ($data=~m/w/i) {
+      while (defined($_=$connection->getline)) {
+        next unless m/\S/;       # blank line
+        if    (m/l/i) {
+          print $connection "$name,L=0 $VERSION lw ".ref($self)."\n";
+        } elsif (m/w/i) {
           $w=$w?0:1;
           print $connection "$name,W=$w\n";
           if ($w) {
-            $watcher_pid=$self->start_watcher($connection);
+            $watcher_pid=$self->start_watcher($connection,$provider);
           } else {
             $self->stop_watcher($watcher_pid);
           }
@@ -74,12 +70,13 @@ sub start {
 sub start_watcher {
   my $self=shift();
   my $fh=shift();
+  my $provider=shift();
   my $pid=fork();
   die("Error: Cannot fork.") unless defined $pid;
   if ($pid) {
     return $pid;
   } else {
-    $self->watcher($fh);
+    $self->watcher($fh, $provider);
   }
 }
 
@@ -90,28 +87,35 @@ sub stop_watcher {
 }
 
 sub watcher {
+  use Time::HiRes qw{time};
   my $self=shift();
   my $fh=shift();
-  
-  use Geo::Forward;
-  use Time::HiRes qw{time};
-  my $object = Geo::Forward->new();
-  my $lasttime=undef;
-  my $time=time;
-  my $lat=$self->lat;
-  my $lon=$self->lon;
-  my $faz=$self->heading;
-  my $speed=$self->speed;
+  my $provider=shift();
+  my $point=undef();
+  my $satellite=undef();
+  my $count=0;
 
   while (1) {
-    $time=time;
-    if (defined $lasttime) {
-      my $dist=$speed * ($lasttime-$time);
-      ($lat,$lon,$faz) = $object->forward($lat,$lon,$faz,$dist);
-      $faz-=180;
+    my $time=time;
+    $point=$provider->point($time, $point);
+    if (ref($point) eq "Net::GPSD::Point") {
+    print $fh $self->name,",O=", 
+      join(" ", $point->tag||"FAKE", $point->time||$time,
+                $point->errortime||0.001, u2q($point->lat), u2q($point->lon),
+                u2q($point->alt), u2q($point->errorhorizontal),
+                u2q($point->errorvertical), u2q($point->heading),
+                u2q($point->speed), u2q($point->climb),
+                u2q($point->errorheading), u2q($point->errorspeed),
+                u2q($point->errorclimb), u2q($point->mode)), "\n";
+    } else {
+      die("Error: provider->point must return Net::GPSD::Point not ". ref($point).".\n");
+    }
+    if ($count++ % 5 == 0) {
+      $satellite=$provider->satellite();
+      if (ref($satellite) eq "Net::GPSD::Satellite") {
+        print ref($satellite), "\n";
+      }
     }
-    print $fh $self->name,",O=FAKE $time 0.005 $lat $lon ? ? ? $faz $speed 0 ? ? ? 2\n";
-    $lasttime=$time;
     sleep 1;
   }
 }
@@ -152,6 +156,10 @@ sub port {
   return $self->{'port'};
 }
 
+sub u2q {
+  my $value=shift();
+  return (!defined($value)||($value eq "")) ? "?" : $value;
+}
 1;
 __END__
 
